"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var test = require("tape");
var shared_functions_1 = require("./shared_functions");
test("test string join ordering", function (assert) {
    shared_functions_1.evaluates(assert, "\n               ~~~\n                 commit\n                      [#foo token:\"a\" level:2]\n                      [#foo token:\"zkp\" level:3]\n                      [#foo token:\"parg\" level:0]\n                      [#foo token:\"naxxo\" level:1]\n               ~~~\n\n                ~~~\n                search\n                  [#foo token level]\n                  index = sort[value:token]                 \n                  a = join[token index given:token with:\"/\"]\n                  a = \"a/naxxo/parg/zkp\"\n                  b = join[token index:level given:token with:\"/\"]\n                  b = \"parg/naxxo/a/zkp\"\n                commit\n                  [#success]\n                ~~~\n  ");
    assert.end();
});
test("check length of hello", function (assert) {
    var expected = {
        insert: [
            ["1", "len", "5"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"hello\"]\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("check length empty string", function (assert) {
    var expected = {
        insert: [
            ["1", "len", "0"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"\"]\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("test length equality", function (assert) {
    var expected = {
        insert: [
            ["1", "len", "4"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"test\"]\n      len = 4\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("test length equality with as", function (assert) {
    var expected = {
        insert: [],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"test\" as: \"symbols\"]\n      len = 3\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("check length of string as symbol", function (assert) {
    var expected = {
        insert: [
            ["1", "len", "5"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"hello\" as: \"symbols\"]\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("nothing is inserted if with invalid as", function (assert) {
    var expected = {
        insert: [],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"hello\" as: \"asdfasdf\"]\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("check length multi-byte characters as symbols", function (assert) {
    var expected = {
        insert: [
            ["1", "len", "2"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"\uD835\uDC01B\" as: \"symbols\"]\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("length of multi-byte characters as code-points", function (assert) {
    var expected = {
        insert: [
            ["1", "len", "3"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      len = length[text: \"\uD835\uDC01b\" as: \"code-points\"]\n\n    commit\n      [len: len]\n    ~~~\n  ");
    assert.end();
});
test("find occurrences in a string", function (assert) {
    var expected = {
        insert: [
            ["a", "at", "3"],
            ["b", "at", "26"],
            ["c", "at", "28"],
            ["d", "at", "34"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      at = find[text: \"I learned to play the Ukulele in Lebanon.\", subtext: \"le\"]\n\n    commit\n      [at]\n    ~~~\n  ");
    assert.end();
});
test("find occurrences in a string case sensitive", function (assert) {
    var expected = {
        insert: [
            ["a", "at", "3"],
            ["b", "at", "26"],
            ["c", "at", "28"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      at = find[text: \"I learned to play the Ukulele in Lebanon.\", subtext: \"le\", case-sensitive: true]\n\n    commit\n      [at]\n    ~~~\n  ");
    assert.end();
});
test("find occurrences in a string with index", function (assert) {
    var expected = {
        insert: [
            ["a", "at", "3"],
            ["a", "ix", "1"],
            ["b", "at", "26"],
            ["b", "ix", "2"],
            ["c", "at", "28"],
            ["c", "ix", "3"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      (at, ix) = find[text: \"I learned to play the Ukulele in Lebanon.\", subtext: \"le\", case-sensitive: true]\n\n    commit\n      [at, ix]\n    ~~~\n  ");
    assert.end();
});
test("find occurrences in a string with index and a starting position", function (assert) {
    var expected = {
        insert: [
            ["b", "at", "26"],
            ["b", "ix", "1"],
            ["c", "at", "28"],
            ["c", "ix", "2"],
        ],
        remove: []
    };
    shared_functions_1.evaluate(assert, expected, "\n    ~~~\n    search\n      (at, ix) = find[text: \"I learned to play the Ukulele in Lebanon.\", subtext: \"le\", case-sensitive: true, from: 20]\n\n    commit\n      [at, ix]\n    ~~~\n  ");
    assert.end();
});
//# sourceMappingURL=strings.js.map